@@ -1,38 +1,39 @@
-/*      $Id: focus.c 22952 2006-08-30 16:44:24Z olivier $
- 
+/*      $Id: focus.c 24170 2006-12-22 20:55:02Z olivier $
+
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
         any later version.
- 
+
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
         GNU General Public License for more details.
- 
+
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- 
+
         oroborus - (c) 2001 Ken Lynch
         xfwm4    - (c) 2002-2006 Olivier Fourdan
- 
+
  */
- 
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#include <X11/X.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
 #include <X11/extensions/shape.h>
-#include <sys/time.h>
+
 #include <glib.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
-#include <libxfce4util/libxfce4util.h> 
+#include <libxfce4util/libxfce4util.h>
 
 #include "screen.h"
 #include "focus.h"
@@ -54,6 +55,7 @@ struct _ClientPair
 
 static Client *client_focus  = NULL;
 static Client *pending_focus = NULL;
+static Client *user_focus    = NULL;
 static Client *last_ungrab   = NULL;
 
 static ClientPair
@@ -72,11 +74,11 @@ clientGetTopMostFocusable (ScreenInfo *screen_info, int layer, Client * exclude)
         TRACE ("*** stack window \"%s\" (0x%lx), layer %i", c->name,
             c->window, (int) c->win_layer);
 
-        if (!clientAcceptFocus (c) || (c->type & (WINDOW_SPLASHSCREEN | WINDOW_DOCK | WINDOW_DESKTOP)))
+        if (!clientAcceptFocus (c) || (c->type & WINDOW_TYPE_DONT_FOCUS))
         {
             continue;
         }
-        
+
         if (!exclude || (c != exclude))
         {
             if ((c->win_layer <= layer) && FLAG_TEST (c->xfwm_flags, XFWM_FLAG_VISIBLE))
@@ -98,7 +100,7 @@ clientGetTopMostFocusable (ScreenInfo *screen_info, int layer, Client * exclude)
 }
 
 void
-clientFocusTop (ScreenInfo *screen_info, int layer)
+clientFocusTop (ScreenInfo *screen_info, int layer, Time timestamp)
 {
     ClientPair top_client;
     DisplayInfo *display_info;
@@ -107,14 +109,14 @@ clientFocusTop (ScreenInfo *screen_info, int layer)
     top_client = clientGetTopMostFocusable (screen_info, layer, NULL);
     if (top_client.prefered)
     {
-        clientSetFocus (screen_info, top_client.prefered, 
-                        myDisplayGetCurrentTime (screen_info->display_info), 
+        clientSetFocus (screen_info, top_client.prefered,
+                        timestamp,
                         NO_FOCUS_FLAG);
     }
     else
     {
-        clientSetFocus (screen_info, top_client.highest, 
-                        myDisplayGetCurrentTime (screen_info->display_info), 
+        clientSetFocus (screen_info, top_client.highest,
+                        timestamp,
                         NO_FOCUS_FLAG);
     }
 }
@@ -129,13 +131,13 @@ clientFocusNew(Client * c)
     gboolean prevented;
 
     g_return_val_if_fail (c != NULL, FALSE);
-    
+
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
     give_focus = (c-> type & WINDOW_REGULAR_FOCUSABLE) && (screen_info->params->focus_new);
     prevent_focus_stealing = screen_info->params->prevent_focus_stealing;
     prevented = FALSE;
-    
+
     /*  Try to avoid focus stealing */
     if (!clientAcceptFocus (c) || (c->type & WINDOW_TYPE_DONT_FOCUS))
     {
@@ -143,7 +145,7 @@ clientFocusNew(Client * c)
     }
     else if ((client_focus) && (prevent_focus_stealing))
     {
-        if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_STARTUP_TIME) && (c->user_time == (Time) 0))
+        if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_STARTUP_TIME) && (c->user_time == (Time) CurrentTime))
         {
             TRACE ("Given startup time is 0, not focusing");
             give_focus = FALSE;
@@ -151,7 +153,7 @@ clientFocusNew(Client * c)
         }
         else if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_STARTUP_TIME | CLIENT_FLAG_HAS_USER_TIME))
         {
-            if (c->user_time < client_focus->user_time)
+            if (TIMESTAMP_IS_BEFORE (c->user_time, client_focus->user_time))
             {
                 TRACE ("Current %u, new %u", (unsigned int) client_focus->user_time, (unsigned int) c->user_time);
                 give_focus = FALSE;
@@ -169,20 +171,17 @@ clientFocusNew(Client * c)
         }
         clientRaise (c, None);
         clientShow (c, TRUE);
-        clientSetFocus (screen_info, c, 
-#if 0
-                        myDisplayGetCurrentTime (display_info), 
-#else
-                        CurrentTime, 
-#endif
+        clientSetFocus (screen_info, c,
+                        myDisplayGetCurrentTime (display_info),
                         FOCUS_IGNORE_MODAL);
     }
     else
     {
         Client *c2 = clientGetFocus();
 
-        if (prevented && c2 && (c2->win_layer == c->win_layer))
+        if ((prevented) && (c2 != NULL) && (c2->win_layer == c->win_layer))
         {
+            TRACE ("clientFocusNew: Setting WM_STATE_DEMANDS_ATTENTION flag on \"%s\" (0x%lx)", c->name, c->window);
             FLAG_SET (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
             clientSortRing(c);
             clientLower (c, c2->frame);
@@ -206,9 +205,8 @@ clientFocusNew(Client * c)
 gboolean
 clientSelectMask (Client * c, int mask, int type)
 {
-    TRACE ("entering clientSelectMask");
-
     g_return_val_if_fail (c != NULL, FALSE);
+    TRACE ("entering clientSelectMask");
 
     if ((!clientAcceptFocus (c)) && !(mask & INCLUDE_SKIP_FOCUS))
     {
@@ -331,7 +329,7 @@ clientPassFocus (ScreenInfo *screen_info, Client *c, Client *exclude)
             {
                 /* If the window is a modal, send focus back to its parent window.
                    Modals are transients, and we aren't interested in modal
-                   for group, so it safe to use clientGetTransient because 
+                   for group, so it safe to use clientGetTransient because
                    it's really what we want...
                  */
                 c2 = clientGetTransient (c);
@@ -342,7 +340,6 @@ clientPassFocus (ScreenInfo *screen_info, Client *c, Client *exclude)
                     clientRaise (c2, None);
                 }
             }
-#if 0
             else
             {
                 c2 = clientGetNext (c, 0);
@@ -351,7 +348,6 @@ clientPassFocus (ScreenInfo *screen_info, Client *c, Client *exclude)
                     new_focus = c2;
                 }
             }
-#endif
         }
     }
     else if (XQueryPointer (myScreenGetXDisplay (screen_info), screen_info->xroot, &dr, &window, &rx, &ry, &wx, &wy, &mask))
@@ -362,12 +358,8 @@ clientPassFocus (ScreenInfo *screen_info, Client *c, Client *exclude)
     {
         new_focus = top_most.prefered ? top_most.prefered : top_most.highest;
     }
-    clientSetFocus (screen_info, new_focus, 
-#if 0
-                    myDisplayGetCurrentTime (screen_info->display_info), 
-#else
-                    CurrentTime, 
-#endif
+    clientSetFocus (screen_info, new_focus,
+                    myDisplayGetCurrentTime (display_info),
                     FOCUS_IGNORE_MODAL | FOCUS_FORCE);
 }
 
@@ -375,20 +367,19 @@ gboolean
 clientAcceptFocus (Client * c)
 {
     g_return_val_if_fail (c != NULL, FALSE);
-
     TRACE ("entering clientAcceptFocus");
 
     /* Modal dialogs *always* accept focus */
     if (FLAG_TEST(c->flags, CLIENT_FLAG_STATE_MODAL))
     {
-        return TRUE; 
+        return TRUE;
     }
     /* First check GNOME protocol */
     if (c->win_hints & WIN_HINTS_SKIP_FOCUS)
     {
         return FALSE;
     }
-    if ((c->screen_info->params->focus_hint) 
+    if ((c->screen_info->params->focus_hint)
         && !FLAG_TEST (c->wm_flags, WM_FLAG_INPUT | WM_FLAG_TAKEFOCUS))
     {
         return FALSE;
@@ -401,12 +392,12 @@ void
 clientSortRing(Client *c)
 {
     ScreenInfo *screen_info;
-    
-    g_return_if_fail (c != NULL);
 
+    g_return_if_fail (c != NULL);
     TRACE ("Sorting...");
+
     screen_info = c->screen_info;
-    if ((screen_info->client_count > 2) && (c != screen_info->clients)) 
+    if ((screen_info->client_count > 2) && (c != screen_info->clients))
     {
         c->prev->next = c->next;
         c->next->prev = c->prev;
@@ -443,7 +434,7 @@ clientUpdateFocus (ScreenInfo *screen_info, Client * c, unsigned short flags)
         return;
     }
 
-    /* 
+    /*
        We can release the button mouse grab if we don't raise on click or if the focused window
        is the one that has been raised at last.
      */
@@ -455,11 +446,18 @@ clientUpdateFocus (ScreenInfo *screen_info, Client * c, unsigned short flags)
     client_focus = c;
     if (c)
     {
+        user_focus = c;
         clientInstallColormaps (c);
         if (flags & FOCUS_SORT)
         {
             clientSortRing(c);
         }
+        if (FLAG_TEST(c->flags, CLIENT_FLAG_DEMANDS_ATTENTION))
+        {
+            TRACE ("Un-setting WM_STATE_DEMANDS_ATTENTION flag on \"%s\" (0x%lx)", c->name, c->window); 
+            FLAG_UNSET (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
+            clientSetNetState (c);
+        }
         data[0] = c->window;
         clientAdjustFullscreenLayer (c, TRUE);
         frameDraw (c, FALSE);
@@ -478,7 +476,7 @@ clientUpdateFocus (ScreenInfo *screen_info, Client * c, unsigned short flags)
         frameDraw (c2, FALSE);
     }
     data[1] = None;
-    XChangeProperty (display_info->dpy, screen_info->xroot, 
+    XChangeProperty (display_info->dpy, screen_info->xroot,
                      display_info->atoms[NET_ACTIVE_WINDOW], XA_WINDOW, 32,
                      PropModeReplace, (unsigned char *) data, 2);
 
@@ -492,7 +490,7 @@ clientSetFocus (ScreenInfo *screen_info, Client * c, Time timestamp, unsigned sh
     Client *c2;
 
     TRACE ("entering clientSetFocus");
-    
+
     display_info = screen_info->display_info;
     c2 = NULL;
     if ((c) && !(flags & FOCUS_IGNORE_MODAL))
@@ -507,18 +505,19 @@ clientSetFocus (ScreenInfo *screen_info, Client * c, Time timestamp, unsigned sh
     c2 = ((client_focus != c) ? client_focus : NULL);
     if ((c) && FLAG_TEST (c->xfwm_flags, XFWM_FLAG_VISIBLE))
     {
-        TRACE ("setting focus to client \"%s\" (0x%lx)", c->name, c->window);
+        TRACE ("setting focus to client \"%s\" (0x%lx) with timestamp %u", c->name, c->window, (unsigned int) timestamp);
+        user_focus = c;
         if (FLAG_TEST(c->flags, CLIENT_FLAG_DEMANDS_ATTENTION))
         {
+            TRACE ("Un-setting WM_STATE_DEMANDS_ATTENTION flag on \"%s\" (0x%lx)", c->name, c->window); 
             FLAG_UNSET (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
             clientSetNetState (c);
         }
         if ((c == client_focus) && !(flags & FOCUS_FORCE))
         {
-            TRACE ("client \"%s\" (0x%lx) is already focused, ignoring request",
-                c->name, c->window);
+            TRACE ("client \"%s\" (0x%lx) is already focused, ignoring request", c->name, c->window);
             return;
-        }        
+        }
         if (!clientAcceptFocus (c))
         {
             TRACE ("SKIP_FOCUS set for client \"%s\" (0x%lx)", c->name, c->window);
@@ -527,7 +526,7 @@ clientSetFocus (ScreenInfo *screen_info, Client * c, Time timestamp, unsigned sh
         if (FLAG_TEST (c->wm_flags, WM_FLAG_INPUT) || !(screen_info->params->focus_hint))
         {
             pending_focus = c;
-            /* 
+            /*
              * When shaded, the client window is unmapped, so it can not be focused.
              * Instead, we focus the frame that is still mapped.
              */
@@ -541,19 +540,28 @@ clientSetFocus (ScreenInfo *screen_info, Client * c, Time timestamp, unsigned sh
             }
             clientUpdateOpacity (screen_info, c);
         }
+        else if (!client_focus)
+        {
+            /* Hack to prevent loosing focus when all remaining windows won't accept focus, see bug #1853 */
+            XSetInputFocus (myScreenGetXDisplay (screen_info), screen_info->xfwm4_win, RevertToPointerRoot, timestamp);
+        }
+
         if (FLAG_TEST(c->wm_flags, WM_FLAG_TAKEFOCUS))
         {
+            pending_focus = c;
             sendClientMessage (c->screen_info, c->window, WM_TAKE_FOCUS, timestamp);
         }
     }
     else
     {
         unsigned long data[2];
-        
+
         TRACE ("setting focus to none");
-        
+
         data[0] = data[1] = None;
         client_focus = NULL;
+        pending_focus = NULL;
+
         if (c2)
         {
             frameDraw (c2, FALSE);
@@ -567,23 +575,85 @@ clientSetFocus (ScreenInfo *screen_info, Client * c, Time timestamp, unsigned sh
     }
 }
 
+void
+clientInitFocusFlag (Client * c)
+{
+    ScreenInfo *screen_info;
+    Client *c2;
+    GList *index;
+    int workspace;
+
+    g_return_if_fail (c != NULL);
+    TRACE ("entering clientSetFocus");
+
+    if (!clientAcceptFocus (c) || (c->type & WINDOW_TYPE_DONT_FOCUS))
+    {
+       return;
+    }
+
+    screen_info = c->screen_info;
+    workspace = c->win_workspace;
+    for (index = screen_info->windows_stack; index; index = g_list_next (index))
+    {
+        c2 = (Client *) index->data;
+        if ((c2->win_workspace == workspace) && FLAG_TEST (c2->xfwm_flags, XFWM_FLAG_FOCUS))
+        {
+            FLAG_UNSET (c2->xfwm_flags, XFWM_FLAG_FOCUS);
+        }
+    }
+    FLAG_SET (c->xfwm_flags, XFWM_FLAG_FOCUS);
+}
+
 Client *
 clientGetFocus (void)
 {
     return (client_focus);
 }
 
-void
-clientClearFocus (void)
+Client *
+clientGetFocusPending (void)
 {
-    client_focus = NULL;
+    return (pending_focus);
+}
+
+Client *
+clientGetFocusOrPending (void)
+{
+    if (client_focus)
+    {
+        return (client_focus);
+    }
+    return (pending_focus);
+}
+
+Client *
+clientGetUserFocus (void)
+{
+    return (user_focus);
+}
+
+void
+clientClearFocus (Client *c)
+{
+    if ((c == NULL) || (c == client_focus))
+    {
+        client_focus = NULL;
+    }
+    if ((c == NULL) || (c == pending_focus))
+    {
+        pending_focus = NULL;
+    }
+    if ((c == NULL) || (c == user_focus))
+    {
+        user_focus = NULL;
+    }
 }
 
 void
 clientGrabMouseButton (Client * c)
 {
     ScreenInfo *screen_info;
-    
+
     g_return_if_fail (c != NULL);
     TRACE ("entering clientGrabMouseButton");
     TRACE ("grabbing buttons for client \"%s\" (0x%lx)", c->name, c->window);
@@ -603,7 +673,7 @@ void
 clientUngrabMouseButton (Client * c)
 {
     ScreenInfo *screen_info;
-    
+
     g_return_if_fail (c != NULL);
     TRACE ("entering clientUngrabMouseButton");
     TRACE ("ungrabing buttons for client \"%s\" (0x%lx)", c->name, c->window);
@@ -619,7 +689,7 @@ clientGrabMouseButtonForAll (ScreenInfo *screen_info)
 {
     Client *c;
     int i;
-    
+
     g_return_if_fail (screen_info != NULL);
     TRACE ("entering clientGrabMouseButtonForAll");
 
@@ -635,7 +705,7 @@ clientUngrabMouseButtonForAll (ScreenInfo *screen_info)
 {
     Client *c;
     int i;
-    
+
     g_return_if_fail (screen_info != NULL);
     TRACE ("entering clientUngrabMouseButtonForAll");
 
