@@ -1,22 +1,22 @@
-/*      $Id: frame.c 22847 2006-08-21 20:13:05Z olivier $
- 
+/*      $Id: frame.c 23925 2006-11-21 20:47:48Z olivier $
+
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
         any later version.
- 
+
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
         GNU General Public License for more details.
- 
+
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- 
+
         oroborus - (c) 2001 Ken Lynch
         xfwm4    - (c) 2002-2006 Olivier Fourdan
- 
+
  */
 
 #ifdef HAVE_CONFIG_H
@@ -27,7 +27,7 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <pango/pango.h>
-#include <libxfce4util/libxfce4util.h> 
+#include <libxfce4util/libxfce4util.h>
 
 #include "screen.h"
 #include "client.h"
@@ -81,7 +81,8 @@ frameLeft (Client * c)
     g_return_val_if_fail (c != NULL, 0);
     if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
         && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN)
-        && !FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED))
+        && (!FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED)
+            || !(c->screen_info->params->borderless_maximize)))
     {
         return c->screen_info->sides[SIDE_LEFT][ACTIVE].width;
     }
@@ -96,7 +97,8 @@ frameRight (Client * c)
     g_return_val_if_fail (c != NULL, 0);
     if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
         && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN)
-        && !FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED))
+        && (!FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED)
+            || !(c->screen_info->params->borderless_maximize)))
     {
         return c->screen_info->sides[SIDE_RIGHT][ACTIVE].width;
     }
@@ -125,7 +127,8 @@ frameBottom (Client * c)
     g_return_val_if_fail (c != NULL, 0);
     if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
         && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN)
-        && !FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED))
+        && (!FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED)
+            || !(c->screen_info->params->borderless_maximize)))
     {
         return c->screen_info->sides[SIDE_BOTTOM][ACTIVE].height;
     }
@@ -140,7 +143,8 @@ frameX (Client * c)
     g_return_val_if_fail (c != NULL, 0);
     if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
         && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN)
-        && !FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED))
+        && (!FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED)
+            || !(c->screen_info->params->borderless_maximize)))
     {
         return c->x - frameLeft (c);
     }
@@ -154,7 +158,7 @@ frameY (Client * c)
 
     g_return_val_if_fail (c != NULL, 0);
     if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
-        && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
+         && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
     {
         return c->y - frameTop (c);
     }
@@ -169,7 +173,8 @@ frameWidth (Client * c)
     g_return_val_if_fail (c != NULL, 0);
     if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
         && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN)
-        && !FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED))
+        && (!FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED)
+            || !(c->screen_info->params->borderless_maximize)))
     {
         return c->width + frameLeft (c) + frameRight (c);
     }
@@ -182,7 +187,7 @@ frameHeight (Client * c)
     TRACE ("entering frameHeight");
 
     g_return_val_if_fail (c != NULL, 0);
-    if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER) 
+    if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
         && FLAG_TEST (c->flags, CLIENT_FLAG_SHADED)
         && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
     {
@@ -199,7 +204,11 @@ frameHeight (Client * c)
 static int
 frameTopLeftWidth (Client * c, int state)
 {
-    if (FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED))
+    TRACE ("entering frameTopLeftWidth");
+
+    g_return_val_if_fail (c != NULL, 0);
+    if (FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED)
+        && c->screen_info->params->borderless_maximize)
     {
         return 0;
     }
@@ -210,7 +219,11 @@ frameTopLeftWidth (Client * c, int state)
 static int
 frameTopRightWidth (Client * c, int state)
 {
-    if (FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED))
+    TRACE ("entering frameTopRightWidth");
+
+    g_return_val_if_fail (c != NULL, 0);
+    if (FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED)
+        && c->screen_info->params->borderless_maximize)
     {
         return 0;
     }
@@ -220,7 +233,11 @@ frameTopRightWidth (Client * c, int state)
 static int
 frameButtonOffset (Client *c)
 {
-    if (FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED))
+    TRACE ("entering frameButtonOffset");
+
+    g_return_val_if_fail (c != NULL, 0);
+    if (FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED)
+        && c->screen_info->params->borderless_maximize)
     {
         return MAX (0, c->screen_info->params->maximized_offset);
     }
@@ -355,11 +372,11 @@ frameCreateTitlePixmap (Client * c, int state, int left, int right, xfwmPixmap *
 
     if (w1 > 0)
     {
-        xfwmPixmapFill (&screen_info->title[TITLE_1][state], pm, 0, 0, w1, frameTop (c)); 
+        xfwmPixmapFill (&screen_info->title[TITLE_1][state], pm, 0, 0, w1, frameTop (c));
         x = x + w1;
     }
 
-    xfwmPixmapFill (&screen_info->title[TITLE_2][state], pm, x, 0, w2, frameTop (c)); 
+    xfwmPixmapFill (&screen_info->title[TITLE_2][state], pm, x, 0, w2, frameTop (c));
     x = x + w2;
 
     if (w3 > 0)
@@ -372,7 +389,7 @@ frameCreateTitlePixmap (Client * c, int state, int left, int right, xfwmPixmap *
         {
             voffset = screen_info->params->title_vertical_offset_inactive;
         }
-        xfwmPixmapFill (&screen_info->title[TITLE_3][state], pm, x, 0, w3, frameTop (c)); 
+        xfwmPixmapFill (&screen_info->title[TITLE_3][state], pm, x, 0, w3, frameTop (c));
         title_x = hoffset + x;
         title_y = voffset + (frameTop (c) - logical_rect.height) / 2;
         if (title_y + logical_rect.height > frameTop (c))
@@ -387,7 +404,7 @@ frameCreateTitlePixmap (Client * c, int state, int left, int right, xfwmPixmap *
             {
                 gdk_draw_layout (gpixmap, gc, title_x + 1, title_y + 1, layout);
             }
-            else 
+            else
             {
                 gdk_draw_layout (gpixmap, gc, title_x - 1, title_y, layout);
                 gdk_draw_layout (gpixmap, gc, title_x, title_y - 1, layout);
@@ -405,12 +422,12 @@ frameCreateTitlePixmap (Client * c, int state, int left, int right, xfwmPixmap *
     {
         x = right - w4;
     }
-    xfwmPixmapFill (&screen_info->title[TITLE_4][state], pm, x, 0, w4, frameTop (c)); 
+    xfwmPixmapFill (&screen_info->title[TITLE_4][state], pm, x, 0, w4, frameTop (c));
     x = x + w4;
 
     if (w5 > 0)
     {
-        xfwmPixmapFill (&screen_info->title[TITLE_5][state], pm, x, 0, w5, frameTop (c)); 
+        xfwmPixmapFill (&screen_info->title[TITLE_5][state], pm, x, 0, w5, frameTop (c));
     }
     g_object_unref (G_OBJECT (gc));
     g_object_unref (G_OBJECT (gpixmap));
@@ -568,7 +585,7 @@ frameSetShape (Client * c, int state, FramePixmap * frame_pix, int button_x[BUTT
 {
     ScreenInfo *screen_info;
     DisplayInfo *display_info;
-    Window temp;
+    Window shape_win;
     XRectangle rect;
     xfwmPixmap *my_pixmap;
     int i;
@@ -584,7 +601,7 @@ frameSetShape (Client * c, int state, FramePixmap * frame_pix, int button_x[BUTT
         return;
     }
 
-    temp = XCreateSimpleWindow (display_info->dpy, c->frame, 0, 0, frameWidth (c), frameHeight (c), 0, 0, 0);
+    shape_win = XCreateSimpleWindow (display_info->dpy, c->frame, 0, 0, frameWidth (c), frameHeight (c), 0, 0, 0);
 
     if (FLAG_TEST (c->flags, CLIENT_FLAG_SHADED))
     {
@@ -592,12 +609,19 @@ frameSetShape (Client * c, int state, FramePixmap * frame_pix, int button_x[BUTT
         rect.y = 0;
         rect.width  = frameWidth (c);
         rect.height = frameHeight (c);
-        XShapeCombineRectangles (display_info->dpy, temp, ShapeBounding, 0, 0, &rect, 1,
-                                 ShapeSubtract, 0);
+        XShapeCombineRectangles (display_info->dpy, shape_win, ShapeBounding, 0, 0, &rect, 1, ShapeSubtract, Unsorted);
+    }
+    else if (!FLAG_TEST (c->flags, CLIENT_FLAG_HAS_SHAPE))
+    {
+        rect.x = frameLeft (c);
+        rect.y = frameTop (c);
+        rect.width  = c->width;
+        rect.height = c->height;
+        XShapeCombineRectangles (display_info->dpy, shape_win, ShapeBounding, 0, 0, &rect, 1, ShapeSet, Unsorted);
     }
     else
     {
-        XShapeCombineShape (display_info->dpy, temp, ShapeBounding, frameLeft (c),
+        XShapeCombineShape (display_info->dpy, shape_win, ShapeBounding, frameLeft (c),
                             frameTop (c), c->window, ShapeBounding, ShapeSet);
     }
     if (frame_pix)
@@ -627,7 +651,7 @@ frameSetShape (Client * c, int state, FramePixmap * frame_pix, int button_x[BUTT
 
         if (xfwmWindowVisible (&c->corners[CORNER_BOTTOM_LEFT]))
         {
-            XShapeCombineMask (display_info->dpy, MYWINDOW_XWINDOW (c->corners[CORNER_BOTTOM_LEFT]), 
+            XShapeCombineMask (display_info->dpy, MYWINDOW_XWINDOW (c->corners[CORNER_BOTTOM_LEFT]),
                                ShapeBounding, 0, 0, screen_info->corners[CORNER_BOTTOM_LEFT][state].mask, ShapeSet);
         }
 
@@ -639,17 +663,17 @@ frameSetShape (Client * c, int state, FramePixmap * frame_pix, int button_x[BUTT
 
         if (xfwmWindowVisible (&c->corners[CORNER_TOP_LEFT]))
         {
-            XShapeCombineMask (display_info->dpy, MYWINDOW_XWINDOW (c->corners[CORNER_TOP_LEFT]), 
+            XShapeCombineMask (display_info->dpy, MYWINDOW_XWINDOW (c->corners[CORNER_TOP_LEFT]),
                                ShapeBounding, 0, 0, screen_info->corners[CORNER_TOP_LEFT][state].mask, ShapeSet);
         }
 
         if (xfwmWindowVisible (&c->corners[CORNER_TOP_RIGHT]))
         {
-            XShapeCombineMask (display_info->dpy, MYWINDOW_XWINDOW (c->corners[CORNER_TOP_RIGHT]), 
+            XShapeCombineMask (display_info->dpy, MYWINDOW_XWINDOW (c->corners[CORNER_TOP_RIGHT]),
                                ShapeBounding, 0, 0, screen_info->corners[CORNER_TOP_RIGHT][state].mask, ShapeSet);
         }
 
-        for (i = 0; i < BUTTON_COUNT; i++)
+        for (i = 0; i < BUTTON_LAST; i++)
         {
             if (xfwmWindowVisible (&c->buttons[i]))
             {
@@ -667,7 +691,7 @@ frameSetShape (Client * c, int state, FramePixmap * frame_pix, int button_x[BUTT
             rect.width  = frameTopLeftWidth (c, state);
             rect.height = screen_info->corners[CORNER_TOP_LEFT][state].height
                            - (frameHeight (c) - frameBottom (c) + 1);
-            XShapeCombineRectangles (display_info->dpy, MYWINDOW_XWINDOW (c->corners[CORNER_TOP_LEFT]), 
+            XShapeCombineRectangles (display_info->dpy, MYWINDOW_XWINDOW (c->corners[CORNER_TOP_LEFT]),
                                      ShapeBounding, 0, 0, &rect, 1, ShapeSubtract, 0);
         }
 
@@ -711,20 +735,20 @@ frameSetShape (Client * c, int state, FramePixmap * frame_pix, int button_x[BUTT
         {
             if (xfwmWindowVisible (&c->sides[SIDE_LEFT]))
             {
-                XShapeCombineShape (display_info->dpy, temp, ShapeBounding, 0, frameTop (c),
+                XShapeCombineShape (display_info->dpy, shape_win, ShapeBounding, 0, frameTop (c),
                                     MYWINDOW_XWINDOW (c->sides[SIDE_LEFT]), ShapeBounding, ShapeUnion);
             }
 
             if (xfwmWindowVisible (&c->sides[SIDE_RIGHT]))
             {
-                XShapeCombineShape (display_info->dpy, temp, ShapeBounding, frameWidth (c) - frameRight (c), frameTop (c),
+                XShapeCombineShape (display_info->dpy, shape_win, ShapeBounding, frameWidth (c) - frameRight (c), frameTop (c),
                                     MYWINDOW_XWINDOW (c->sides[SIDE_RIGHT]), ShapeBounding, ShapeUnion);
             }
         }
 
         if (xfwmWindowVisible (&c->title))
         {
-            XShapeCombineShape (display_info->dpy, temp, ShapeBounding,
+            XShapeCombineShape (display_info->dpy, shape_win, ShapeBounding,
                                 frameTopLeftWidth (c, state), 0,
                                 MYWINDOW_XWINDOW (c->title), ShapeBounding, ShapeUnion);
         }
@@ -732,13 +756,13 @@ frameSetShape (Client * c, int state, FramePixmap * frame_pix, int button_x[BUTT
         if (xfwmWindowVisible (&c->corners[CORNER_TOP_LEFT]))
         {
 
-            XShapeCombineShape (display_info->dpy, temp, ShapeBounding, 0, 0,
+            XShapeCombineShape (display_info->dpy, shape_win, ShapeBounding, 0, 0,
                                 MYWINDOW_XWINDOW (c->corners[CORNER_TOP_LEFT]), ShapeBounding, ShapeUnion);
         }
 
         if (xfwmWindowVisible (&c->sides[SIDE_BOTTOM]))
         {
-            XShapeCombineShape (display_info->dpy, temp, ShapeBounding,
+            XShapeCombineShape (display_info->dpy, shape_win, ShapeBounding,
                                 screen_info->corners[CORNER_BOTTOM_LEFT][state].width,
                                 frameHeight (c) - frameBottom (c),
                                 MYWINDOW_XWINDOW (c->sides[SIDE_BOTTOM]), ShapeBounding, ShapeUnion);
@@ -746,14 +770,14 @@ frameSetShape (Client * c, int state, FramePixmap * frame_pix, int button_x[BUTT
 
         if (xfwmWindowVisible (&c->corners[CORNER_BOTTOM_LEFT]))
         {
-            XShapeCombineShape (display_info->dpy, temp, ShapeBounding, 0,
+            XShapeCombineShape (display_info->dpy, shape_win, ShapeBounding, 0,
                                 frameHeight (c) - screen_info->corners[CORNER_BOTTOM_LEFT][state].height,
                                 MYWINDOW_XWINDOW (c->corners[CORNER_BOTTOM_LEFT]), ShapeBounding, ShapeUnion);
         }
 
         if (xfwmWindowVisible (&c->corners[CORNER_BOTTOM_RIGHT]))
         {
-            XShapeCombineShape (display_info->dpy, temp, ShapeBounding,
+            XShapeCombineShape (display_info->dpy, shape_win, ShapeBounding,
                                 frameWidth (c) - screen_info->corners[CORNER_BOTTOM_RIGHT][state].width,
                                 frameHeight (c) - screen_info->corners[CORNER_BOTTOM_RIGHT][state].height,
                                 MYWINDOW_XWINDOW (c->corners[CORNER_BOTTOM_RIGHT]), ShapeBounding, ShapeUnion);
@@ -761,30 +785,29 @@ frameSetShape (Client * c, int state, FramePixmap * frame_pix, int button_x[BUTT
 
         if (xfwmWindowVisible (&c->corners[CORNER_TOP_RIGHT]))
         {
-            XShapeCombineShape (display_info->dpy, temp, ShapeBounding,
+            XShapeCombineShape (display_info->dpy, shape_win, ShapeBounding,
                                 frameWidth (c) - frameTopRightWidth (c, state),
                                 0, MYWINDOW_XWINDOW (c->corners[CORNER_TOP_RIGHT]), ShapeBounding, ShapeUnion);
         }
-        
-        for (i = 0; i < BUTTON_COUNT; i++)
+
+        for (i = 0; i < BUTTON_LAST; i++)
         {
             if (xfwmWindowVisible (&c->buttons[i]))
             {
-                XShapeCombineShape (display_info->dpy, temp, ShapeBounding, button_x[i],
+                XShapeCombineShape (display_info->dpy, shape_win, ShapeBounding, button_x[i],
                                     (frameTop (c) - screen_info->buttons[i][state].height + 1) / 2,
                                     MYWINDOW_XWINDOW (c->buttons[i]), ShapeBounding, ShapeUnion);
             }
         }
     }
-
     rect.x = 0;
     rect.y = 0;
     rect.width  = frameWidth (c);
     rect.height = frameHeight (c);
-    XShapeCombineRectangles (display_info->dpy, temp, ShapeBounding, 0, 0, &rect, 1, ShapeIntersect, 0);
-    XShapeCombineShape (display_info->dpy, c->frame, ShapeBounding, 0, 0, temp, ShapeBounding, ShapeSet);
+    XShapeCombineRectangles (display_info->dpy, shape_win, ShapeBounding, 0, 0, &rect, 1, ShapeIntersect, Unsorted);
+    XShapeCombineShape (display_info->dpy, c->frame, ShapeBounding, 0, 0, shape_win, ShapeBounding, ShapeSet);
 
-    XDestroyWindow (display_info->dpy, temp);
+    XDestroyWindow (display_info->dpy, shape_win);
 }
 
 void
@@ -831,7 +854,7 @@ frameDraw (Client * c, gboolean clear_all)
         }
     }
 
-    if ((state == INACTIVE) 
+    if ((state == INACTIVE)
         && FLAG_TEST(c->xfwm_flags, XFWM_FLAG_DRAW_ACTIVE | XFWM_FLAG_FIRST_MAP))
     {
         requires_clearing = TRUE;
@@ -872,7 +895,7 @@ frameDraw (Client * c, gboolean clear_all)
         && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
     {
         /* First, hide the buttons that we don't have... */
-        for (i = 0; i < BUTTON_COUNT; i++)
+        for (i = 0; i < BUTTON_LAST; i++)
         {
             char b = getLetterFromButton (i, c);
             if ((!b) || !strchr (screen_info->params->button_layout, b))
@@ -980,17 +1003,18 @@ frameDraw (Client * c, gboolean clear_all)
         /* Corners are never resized, we need to update them separately */
         if (requires_clearing)
         {
-            xfwmWindowSetBG (&c->corners[CORNER_TOP_LEFT], 
+            xfwmWindowSetBG (&c->corners[CORNER_TOP_LEFT],
                 &screen_info->corners[CORNER_TOP_LEFT][state]);
-            xfwmWindowSetBG (&c->corners[CORNER_TOP_RIGHT], 
+            xfwmWindowSetBG (&c->corners[CORNER_TOP_RIGHT],
                 &screen_info->corners[CORNER_TOP_RIGHT][state]);
-            xfwmWindowSetBG (&c->corners[CORNER_BOTTOM_LEFT], 
+            xfwmWindowSetBG (&c->corners[CORNER_BOTTOM_LEFT],
                 &screen_info->corners[CORNER_BOTTOM_LEFT][state]);
-            xfwmWindowSetBG (&c->corners[CORNER_BOTTOM_RIGHT], 
+            xfwmWindowSetBG (&c->corners[CORNER_BOTTOM_RIGHT],
                 &screen_info->corners[CORNER_BOTTOM_RIGHT][state]);
         }
 
-        if (FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED))
+        if (FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED)
+            && (c->screen_info->params->borderless_maximize))
         {
             xfwmWindowHide (&c->sides[SIDE_LEFT]);
             xfwmWindowHide (&c->sides[SIDE_RIGHT]);
@@ -1011,20 +1035,20 @@ frameDraw (Client * c, gboolean clear_all)
             {
                 xfwmPixmapCreate (screen_info, &frame_pix.pm_sides[SIDE_LEFT],
                     frameLeft (c), left_height);
-                xfwmPixmapFill (&screen_info->sides[SIDE_LEFT][state], 
+                xfwmPixmapFill (&screen_info->sides[SIDE_LEFT][state],
                     &frame_pix.pm_sides[SIDE_LEFT],
-                    0, 0, frameLeft (c), left_height); 
-                xfwmWindowSetBG (&c->sides[SIDE_LEFT], 
+                    0, 0, frameLeft (c), left_height);
+                xfwmWindowSetBG (&c->sides[SIDE_LEFT],
                     &frame_pix.pm_sides[SIDE_LEFT]);
                 xfwmWindowShow (&c->sides[SIDE_LEFT], 0, frameTop (c),
                     frameLeft (c), left_height, (requires_clearing | height_changed));
 
                 xfwmPixmapCreate (screen_info, &frame_pix.pm_sides[SIDE_RIGHT],
                     frameRight (c), right_height);
-                xfwmPixmapFill (&screen_info->sides[SIDE_RIGHT][state], 
-                    &frame_pix.pm_sides[SIDE_RIGHT], 
-                    0, 0, frameRight (c), right_height); 
-                xfwmWindowSetBG (&c->sides[SIDE_RIGHT], 
+                xfwmPixmapFill (&screen_info->sides[SIDE_RIGHT][state],
+                    &frame_pix.pm_sides[SIDE_RIGHT],
+                    0, 0, frameRight (c), right_height);
+                xfwmWindowSetBG (&c->sides[SIDE_RIGHT],
                     &frame_pix.pm_sides[SIDE_RIGHT]);
                 xfwmWindowShow (&c->sides[SIDE_RIGHT],
                     frameWidth (c) - frameRight (c), frameTop (c), frameRight (c),
@@ -1033,10 +1057,10 @@ frameDraw (Client * c, gboolean clear_all)
 
             xfwmPixmapCreate (screen_info, &frame_pix.pm_sides[SIDE_BOTTOM],
                 bottom_width, frameBottom (c));
-            xfwmPixmapFill (&screen_info->sides[SIDE_BOTTOM][state], 
-                &frame_pix.pm_sides[SIDE_BOTTOM], 
+            xfwmPixmapFill (&screen_info->sides[SIDE_BOTTOM][state],
+                &frame_pix.pm_sides[SIDE_BOTTOM],
                 0, 0, bottom_width, frameBottom (c));
-            xfwmWindowSetBG (&c->sides[SIDE_BOTTOM], 
+            xfwmWindowSetBG (&c->sides[SIDE_BOTTOM],
                 &frame_pix.pm_sides[SIDE_BOTTOM]);
             xfwmWindowShow (&c->sides[SIDE_BOTTOM],
                 screen_info->corners[CORNER_BOTTOM_LEFT][state].width,
@@ -1096,7 +1120,7 @@ frameDraw (Client * c, gboolean clear_all)
                 xfwmWindowHide (&c->corners[i]);
             }
         }
-        for (i = 0; i < BUTTON_COUNT; i++)
+        for (i = 0; i < BUTTON_LAST; i++)
         {
             if (MYWINDOW_XWINDOW (c->buttons[i]) && xfwmWindowVisible (&c->buttons[i]))
             {
